﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.System;
using Windows.System.Threading;
using Windows.UI;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.UI.Xaml.Shapes;

//https://go.microsoft.com/fwlink/?LinkId=234236 上介绍了“用户控件”项模板

namespace tetrisuwp
{
    public sealed partial class TetrisBoard : UserControl
    {
        ThreadPoolTimer moveTimer;
        //ThreadPoolTimer renderTimer;
        int level = 1;
        int removed = 0;
        int score = 0;
        int levelScore = 0;
        int ticks = 1000;

        public TetrisShape Shape { get; set; }

        private bool[,] datas = new bool[20, 10];
        private Rectangle[,] cubes = new Rectangle[20, 10];
        private Rectangle[,] previews = new Rectangle[4, 4];

        public TetrisBoard()
        {
            this.InitializeComponent();

            this.Shape = new TetrisShape();
            Shape.GridData = datas;

            Shape.Moved += () =>
            {
                Render();
            };

            Shape.CreatedNew += () =>
            {
                for (var row = 0; row < 4; row++)
                {
                    for (var col = 0; col < 4; col++)
                    {
                        previews[row, col].Visibility = Visibility.Collapsed;
                    }
                }

                var shape = Shape.Next;

                var data = shape.Data;

                var len = data.GetLength(1);

                var left = 4 - len;

                for (var row = 0; row < data.GetLength(0); row++)
                {
                    for (var col = 0; col < len; col++)
                    {
                        if (data[row, col])
                        {
                            previews[row, left + col].Visibility = Visibility.Visible;
                            previews[row, left + col].Fill = new SolidColorBrush(shape.Color);
                        }
                    }
                }
            };

            Shape.ReachedBottom += () =>
            {
                if (Shape.Preview != null)
                {
                    var arr = Shape.Preview.Data;

                    for (int row = 0; row < arr.GetLength(0); row++)
                    {
                        for (var col = 0; col < arr.GetLength(1); col++)
                        {
                            if (arr[row, col])
                            {
                                var cube = cubes[Shape.Preview.Top + row, Shape.Preview.Left + col];
                                cube.Fill = new SolidColorBrush(Shape.Preview.Color);
                                cube.Visibility = Visibility.Visible;

                            }
                        }
                    }
                }

                Remove();

                rowsText.Text = removed.ToString();
                scoreText.Text = score.ToString();
                levelText.Text = level.ToString();

                if (Shape.IsFull())
                {
                    moveTimer.Cancel();
                    ContentDialog noWifiDialog = new ContentDialog
                    {
                        Title = "Tips",
                        Content = "Game over.",
                        CloseButtonText = "Ok"
                    };

                    noWifiDialog.ShowAsync();
                }

                Render();
            };

        }
        public void Remove()
        {
            int removeCount = 0;
            for (var row = 0; row < 20; row++)
            {
                bool full = true;
                for (var col = 0; col < 10; col++)
                {
                    full = full && datas[row, col];
                }
                if (full)
                {
                    removed++;
                    removeCount++;
                    for (var ui = row - 1; ui > 0; ui--)
                    {
                        for (var uj = 0; uj < 10; uj++)
                        {
                            datas[ui + 1, uj] = datas[ui, uj]; //后一行等于前一行。
                        }
                    }
                }
            }

            var tempScore = 0;
            if (removeCount == 1) tempScore += 100;
            if (removeCount == 2) tempScore += 200;
            if (removeCount == 3) tempScore += 400;
            if (removeCount == 4) tempScore += 400;

            score += tempScore;
            levelScore += tempScore;

            if (levelScore >= 4000)
            {
                if (level < 12)
                    level++;
                else
                    level = 1;

                levelScore = 0;
            }
        }

        void CreateTimer()
        {
            if (moveTimer != null)
            {
                moveTimer.Cancel();
            }
            moveTimer = ThreadPoolTimer.CreatePeriodicTimer(async (t) =>
           {
               await Dispatcher.RunAsync(CoreDispatcherPriority.High, () =>
               {
                   Shape.MoveDown();

               });
           }, TimeSpan.FromMilliseconds(ticks));
        }

        public void ProcessKey(VirtualKey key)
        {
            if (key == VirtualKey.Left) { Shape.MoveLeft(); }
            if (key == VirtualKey.Right) { Shape.MoveRight(); }
            if (key == VirtualKey.End) { Shape.Drop(); }
            if (key == VirtualKey.Up) { Shape.ChangeDirection(); }
            if (key == VirtualKey.Down) { Shape.MoveQuickly(); }
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {

            for (var col = 0; col <= 10; col++)
            {
                Line line = new Line();
                line.X1 = col * 30;
                line.X2 = col * 30;
                line.Y1 = 0;
                line.Y2 = 600;
                line.Stroke = new SolidColorBrush(Color.FromArgb(0xFF, 0, 0, 1));
                grid.Children.Add(line);
            }

            for (var row = 0; row <= 20; row++)
            {
                Line line = new Line();
                line.X1 = 0;
                line.X2 = 300;
                line.Y1 = row * 30;
                line.Y2 = row * 30;
                line.Stroke = new SolidColorBrush(Color.FromArgb(0xFF, 0, 0, 0xFF));
                grid.Children.Add(line);
            }

            for (int row = 0; row < 4; row++)
            {
                for (int col = 0; col < 4; col++)
                {
                    Rectangle cube = new Rectangle();
                    cube.Width = 22;
                    cube.Height = 22;
                    cube.HorizontalAlignment = HorizontalAlignment.Left;
                    cube.VerticalAlignment = VerticalAlignment.Top;

                    cube.Visibility = Visibility.Collapsed;
                    var m = cube.Margin;
                    cube.Margin = new Thickness(m.Left + col * 25 + 3, m.Top + row * 25 + 3, m.Right, m.Bottom);
                    previews[row, col] = cube;
                    next.Children.Add(cube);
                }
            }
            for (int row = 0; row < 20; row++)
            {
                for (int col = 0; col < 10; col++)
                {
                    bool show = datas[row, col];
                    Rectangle cube = new Rectangle();
                    cube.Width = 24;
                    cube.Height = 24;
                    cube.HorizontalAlignment = HorizontalAlignment.Left;
                    cube.VerticalAlignment = VerticalAlignment.Top;
                    if (show)
                    {
                        cube.Visibility = Visibility.Visible;
                    }
                    else
                    {
                        cube.Visibility = Visibility.Collapsed;
                    }
                    var m = cube.Margin;
                    cube.Margin = new Thickness(m.Left + col * 30 + 3, m.Top + row * 30 + 3, m.Right, m.Bottom);
                    cubes[row, col] = cube;
                    grid.Children.Add(cube);
                }
            }
        }



        protected async void Render()
        {
            for (int row = 0; row < 20; row++)
            {
                for (int col = 0; col < 10; col++)
                {
                    bool show = datas[row, col];
                    Rectangle cube = cubes[row, col];
                    if (show)
                    {
                        cube.Visibility = Visibility.Visible;
                    }
                    else
                    {
                        cube.Visibility = Visibility.Collapsed;
                    }
                }
            }

            if (Shape.Shape == null) return;
            var arr = Shape.Shape.Data;

            for (int row = 0; row < arr.GetLength(0); row++)
            {
                for (var col = 0; col < arr.GetLength(1); col++)
                {
                    if (arr[row, col])
                    {
                        var cube = cubes[Shape.Top + row, Shape.Left + col];
                        cube.Fill = new SolidColorBrush(Shape.ShapeColor);
                        cube.Visibility = Visibility.Visible;
                    }

                }
            }
        }

        public void Upgrade()
        {
            if (level < 12)
                level++;
            else
                level = 1;

            ticks = 1000 / level;

            CreateTimer();
            //timer.Interval = ticks;
        }

        public void Start()
        {
            for (var row = 0; row < 20; row++)
            {
                for (var col = 0; col < 10; col++)
                {
                    this.datas[row, col] = false;
                }
            }

            Shape.RandomShape();
            //timer.Start();
            CreateTimer();

        }

        public void Stop()
        {
            if (moveTimer != null)
                moveTimer.Cancel();
        }

        public void Pause()
        {
            if (moveTimer != null)
            {
                moveTimer.Cancel();
                CreateTimer();
            }
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            scoreText.Text = "0";
            rowsText.Text = "0";
            levelText.Text = "0";
            Start();
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            Pause();
        }
    }
}
